﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Drawing.Imaging;

namespace howto_split_images
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            string src_dir = Path.Combine(Application.StartupPath, "..\\..");
            DirectoryInfo dir_info = new DirectoryInfo(src_dir);
            txtFromDir.Text = dir_info.FullName + "\\Original Images";
            txtToDir.Text = dir_info.FullName + "\\Split Images";
        }

        // Process the image files in the input directory,
        // saving the results in the output directory.
        private void btnSplit_Click(object sender, EventArgs e)
        {
            string from_dir, to_dir;
            from_dir = txtFromDir.Text;
            to_dir = txtToDir.Text;
            bool split_horizontally = radSplitHorizontally.Checked;
            bool remove_whitespace = chkRemoveWhitespace.Checked;

            foreach (string filename in Directory.GetFiles(from_dir))
            {
                FileInfo file_info = new FileInfo(filename);
                string extension = file_info.Extension.ToLower();
                if ((extension == ".png") ||
                    (extension == ".jpg") ||
                    (extension == ".gif") ||
                    (extension == ".tiff") ||
                    (extension == ".jpeg"))
                {
                    SplitFile(file_info, to_dir,
                        split_horizontally,
                        remove_whitespace);
                }
            }
            picSample.Image = null;
        }

        // Split the image file into two pieces.
        private void SplitFile(FileInfo file_info,
            string to_dir, bool split_horizontally,
            bool remove_whitespace)
        {
            Bitmap bm = LoadBitmapUnlocked(file_info.FullName);
            picSample.Image = bm;
            picSample.Refresh();

            int wid = bm.Width / 2;
            int hgt = bm.Height / 2;
            if (split_horizontally)
                hgt = bm.Height;
            else
                wid = bm.Width;

            Rectangle src_rect_a = new Rectangle(0, 0, wid, hgt);
            Bitmap bm_a = new Bitmap(wid, hgt);
            using (Graphics gr = Graphics.FromImage(bm_a))
            {
                gr.DrawImage(bm, 0, 0, src_rect_a,  GraphicsUnit.Pixel);
            }
            if (remove_whitespace) bm_a = RemoveWhitespace(bm_a);
            string filename_a = to_dir + "\\" +
                Path.GetFileNameWithoutExtension(file_info.Name) +
                "_a" + file_info.Extension;
            SaveImage(bm_a, filename_a);

            Rectangle src_rect_b;
            if (split_horizontally)
                src_rect_b = new Rectangle(wid, 0, wid, hgt);
            else
                src_rect_b = new Rectangle(0, hgt, wid, hgt);
            Bitmap bm_b = new Bitmap(wid, hgt);
            using (Graphics gr = Graphics.FromImage(bm_b))
            {
                gr.DrawImage(bm, 0, 0, src_rect_b, GraphicsUnit.Pixel);
            }
            if (remove_whitespace) bm_b = RemoveWhitespace(bm_b);
            string filename_b = to_dir + "\\" +
                Path.GetFileNameWithoutExtension(file_info.Name) +
                "_b" + file_info.Extension;
            SaveImage(bm_b, filename_b);           
        }

        // Return a copy of the parts of the bitmap that is not white.
        private Bitmap RemoveWhitespace(Bitmap bm)
        {
            Rectangle src_rect = NonWhiteBounds(bm);
            Rectangle dest_rect = new Rectangle(0, 0,
                src_rect.Width, src_rect.Height);

            Bitmap new_bm = new Bitmap(src_rect.Width, src_rect.Height);
            using (Graphics gr = Graphics.FromImage(new_bm))
            {
                gr.DrawImage(bm, dest_rect, src_rect, GraphicsUnit.Pixel);
            }
            return new_bm;
        }

        // Find the bounds of the image's non-white pixels.
        private Rectangle NonWhiteBounds(Bitmap bm)
        {
            Bitmap32 bm32 = new Bitmap32(bm);
            bm32.LockBitmap();

            int ymin = bm.Height;
            for (int y = 0; y < bm.Height; y++)
            {
                for (int x = 0; x < bm.Width; x++)
                {
                    byte r, g, b, a;
                    bm32.GetPixel(x, y, out r, out g, out b, out a);
                    if (r + g + b < 765)
                    {
                        ymin = y;
                        break;
                    }
                }
                if (ymin < bm.Height) break;
            }
            if (ymin == bm.Height) return new Rectangle(0, 0, -1, -1);

            int ymax = -1;
            for (int y = bm.Height - 1; y >= ymin; y--)
            {
                for (int x = 0; x < bm.Width; x++)
                {
                    byte r, g, b, a;
                    bm32.GetPixel(x, y, out r, out g, out b, out a);
                    if (r + g + b < 765)
                    {
                        ymax = y;
                        break;
                    }
                }
                if (ymax > -1) break;
            }

            int xmin = bm.Width - 1;
            for (int x = 0; x < bm.Width; x++)
            {
                for (int y = ymin; y <= ymax; y++)
                {
                    byte r, g, b, a;
                    bm32.GetPixel(x, y, out r, out g, out b, out a);
                    if (r + g + b < 765)
                    {
                        xmin = x;
                        break;
                    }
                }
                if (xmin < bm.Width - 1) break;
            }

            int xmax = -1;
            for (int x = bm.Width - 1; x >= xmin; x--)
            {
                for (int y = ymin; y <= ymax; y++)
                {
                    byte r, g, b, a;
                    bm32.GetPixel(x, y, out r, out g, out b, out a);
                    if (r + g + b < 765)
                    {
                        xmax = x;
                        break;
                    }
                }
                if (xmax > -1) break;
            }

            bm32.UnlockBitmap();

            return new Rectangle(xmin, ymin,
                xmax - xmin + 1,
                ymax - ymin + 1);
        }

        // Load a bitmap without locking it.
        private Bitmap LoadBitmapUnlocked(string file_name)
        {
            using (Bitmap bm = new Bitmap(file_name))
            {
                return new Bitmap(bm);
            }
        }

        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }
    }
}
